==> 0.4.72.tar.gz: lib/picker_range.dart <==
import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/model.dart';
import 'package:wowit/sql_service.dart';
import 'common.dart';
import 'methods.dart';
import 'package:collection/collection.dart';

class PickerRange {
  PickerRange._();
  static const String _picker = 'picker';
  static const String _range = 'range';

  static const String _splitTable = '[::]';
  static const String _splitValue = '[..]';
  static const String _splitData = '[,,]';

  static KeyValue getDefaultPicker({required String tableName}) {
    if (kDebugMode) {
      return KeyValue(
          -1, -1, 'Chưa nhập liệu cho drop dơn này: ' + tableName.toString());
    } else {
      return const KeyValue(-1, -1, '--');
    }
  }

  static init(
    Map<String, List<Map<String, Object?>?>?> reponse, {
    bool reload = false,
  }) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> pickers = prefs.getStringList(_picker) ?? [];
    List<String> ranges = prefs.getStringList(_range) ?? [];
    if (pickers.isEmpty || ranges.isEmpty) {
      reload = true;
    }
    if (reload) {
      try {
        if (reponse.isEmpty) {
          reponse = await SQLService.execute('sp_InitSetting',
              params: {'@getPickerRange': true});
          if (reponse.isEmpty) {
            return;
          }
        }
        reponse.forEach((key, value) {
          String tableName = key;
          if (kDebugMode) {
            print('Picker-Range init: ' + tableName);
          }
          if (tableName.startsWith('DataRange_')) {
            PickerRange.initDataRange(
                tableName.replaceAll('DataRange_', ''), value);
          } else if (tableName.startsWith('DataPicker_')) {
            PickerRange.initDataPicker(
                tableName.replaceAll('DataPicker_', ''), value);
          }
        });
        savePickerToLocal();
        saveRangeToLocal();
        await SQLService.execute('sp_updatePicker');
        return;
      } catch (ex) {
        if (kDebugMode) {
          print('PickerRange init exception: ' + ex.toString());
        }
      }
    } else {
      loadPickerFromLocal(pickers: pickers);
      loadRangeFromLocal(ranges: ranges);
    }
  }

  static String getTitle(
    String tableName,
    int? id, {
    String defaultValue = '',
  }) {
    if (id == null) return defaultValue;
    return getDataPicker(
      getValueDataPicker(tableName),
      id,
      defaultValue: defaultValue,
    );
  }

  static String getDataPicker(
    List<KeyValue> data,
    int? id, {
    String defaultValue = '',
  }) {
    if (id == null) return defaultValue;
    return data.firstWhereOrNull((e) => e.valueMember == id)?.displayMember ??
        defaultValue;
    // for (int i = 0; i < data.length; i++) {
    //   try {
    //     if (data[i].valueMember == id) {
    //       return data[i].displayMember;
    //     }
    //   } catch (e) {
    //     if (kDebugMode) {
    //       HandleException.log(e, subTitle: 'getDataPicker');
    //       return defaultValue;
    //     }
    //   }
    // }

    // return defaultValue;
  }

  static List<KeyValue> getValueDataPicker(
    String tableName, {
    int? parentId,
  }) {
    if (Common.mapPicker.containsKey(tableName)) {
      if (parentId == null || parentId < 1) {
        return Common.mapPicker[tableName] ?? [];
      }
      List<KeyValue> data = Common.mapPicker[tableName] ?? [];
      List<KeyValue> lst = [];
      for (int i = 0; i < data.length; i++) {
        if (data[i].parentValue == parentId) {
          lst.add(data[i]);
        }
      }
      if (lst.isEmpty) {
        lst.add(PickerRange.getDefaultPicker(tableName: tableName));
        if (kDebugMode) {
          SQLService.execute('sp_AddPickerRange',
              params: {'@tableName': tableName});
        }
      }
      return lst;
    } else {
      if (kDebugMode) {
        SQLService.execute(
          'sp_AddPickerRange',
          params: {
            '@tableName': tableName,
          },
        );
      }
    }
    return [
      PickerRange.getDefaultPicker(tableName: tableName),
    ];
  }

  static Future<List<KeyValue>> initDataPicker(
    String tableName,
    List<Map<String, Object?>?>? dtData, {
    bool override = false,
  }) async {
    if (Common.mapPicker.containsKey(tableName) && !override) {
      return Common.mapPicker[tableName] ?? [];
    }
    List<KeyValue> listVal = [];
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute(
          'sp_getValueDataPicker',
          params: {
            '@tableName': tableName,
          },
        );
        if (ds.isEmpty) return listVal;
        dtData = ds['sp_getValueDataPicker'] ?? ds.values.first;
      }
      dtData ??= [];

      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        int parentId = Methods.getInt(row, 'parentId');
        String title = Methods.getString(row, 'title');
        if (id < 0) continue;
        listVal.add(KeyValue(parentId, id, title));
      }
      Common.mapPicker[tableName] = listVal;
    } catch (ex) {
      if (kDebugMode) {
        print('[Exception] getValueDataPicker: ' + ex.toString());
      }
    }
    if (kDebugMode) {
      print('getValueDataPicker: ' +
          tableName +
          ' count: ' +
          listVal.length.toString());
    }
    return listVal;
  }

  static Map<int, RangeValue> getValueDataRange(String tableName) {
    if (Common.mapRange.containsKey(tableName)) {
      return Common.mapRange[tableName] ?? {};
    } else if (kDebugMode) {
      SQLService.execute('sp_AddPickerRange',
          params: {'@tableName': tableName});
    }
    return {};
  }

  static Future<Map<int, RangeValue>> initDataRange(
    String tableName,
    List<Map<String, Object?>?>? dtData, {
    bool override = false,
  }) async {
    if (Common.mapRange.containsKey(tableName) && !override) {
      return Common.mapRange[tableName] ?? {};
    }
    Map<int, RangeValue> listVal = {};
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute('sp_getValueDataRange',
            params: {'@tableName': tableName});
        if (ds.isEmpty) return listVal;
        dtData = ds['sp_getValueDataRange'] ?? ds.values.first;
      }
      dtData ??= [];
      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        String title = Methods.getString(row, 'title');
        double from = Methods.getDouble(row, 'valFrom', defaultValue: 0);
        double to = Methods.getDouble(row, 'valTo', defaultValue: 0);
        if (id < 0) continue;
        listVal[id] = RangeValue(from, to, title: title);
      }
      Common.mapRange[tableName] = listVal;
    } catch (ex) {
      if (kDebugMode) {
        print('[Exception] initDataRange: ' + ex.toString());
      }
    }
    if (kDebugMode) {
      print('getValueDataPicker: ' +
          tableName +
          ' count: ' +
          listVal.length.toString());
    }
    return listVal;
  }

  static RangeValue getRangeValueDataPicker(
      Map<int, RangeValue> rangeMap, int id,
      {RangeValue defaultValue = const RangeValue(0, 0)}) {
    return rangeMap[id] ?? defaultValue;
  }

  static void savePickerToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> pickers = Common.mapPicker.entries.map<String>((e) {
      return e.key +
          _splitTable +
          e.value.map<String>((e) {
            String parentValue = e.parentValue.toString();
            String valueMember = e.valueMember.toString();
            String displayMember =
                e.displayMember.isNotEmpty ? e.displayMember : ' ';
            return parentValue +
                _splitValue +
                valueMember +
                _splitValue +
                displayMember;
          }).join(_splitData);
    }).toList();

    await prefs.setStringList(_picker, pickers);
    if (kDebugMode) {
      print('save Picker to local success');
    }
  }

  static void saveRangeToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> ranges = Common.mapRange.entries.map<String>((e) {
      return e.key +
          _splitTable +
          e.value.entries.map<String>((e) {
            String id = e.key.toString();
            String start = e.value.start.toString();
            String end = e.value.start.toString();
            String title = e.value.title;
            title = title.isNotEmpty ? title : ' ';
            return id +
                _splitValue +
                start +
                _splitValue +
                end +
                _splitValue +
                title;
          }).join(_splitData);
    }).toList();

    await prefs.setStringList(_range, ranges);
    if (kDebugMode) {
      print('save Range to local success');
    }
  }

  static void loadPickerFromLocal({List<String>? pickers}) async {
    if (pickers == null || pickers.isEmpty) {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      pickers = prefs.getStringList(_picker) ?? [];
    }
    Common.mapPicker = Map.fromIterable(
      pickers,
      key: (e) {
        String key = e as String;
        return key.split(_splitTable).first;
      },
      value: (e) {
        String dataset = e as String;
        return dataset.split(_splitTable).last.split(_splitData).map((e) {
          List<String> splits = e.split(_splitValue);
          int parentValue = Methods.toInt(splits[0]);
          int valueMember = Methods.toInt(splits[1]);
          String displayMember = splits[2];
          return KeyValue(parentValue, valueMember, displayMember);
        }).toList();
      },
    );
    if (kDebugMode) {
      print('//==============================');
      print('||load Picker from local success');
      Common.mapPicker.forEach((key, value) {
        if (kDebugMode) {
          print('||$key: ${value.length} item');
        }
      });
      print('\\\\==============================');
    }
  }

  static void loadRangeFromLocal({List<String>? ranges}) async {
    if (ranges == null || ranges.isEmpty) {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      ranges = prefs.getStringList(_range) ?? [];
    }
    Common.mapRange = Map.fromIterable(
      ranges,
      key: (e) {
        String key = e as String;
        return key.split(_splitTable).first;
      },
      value: (e) {
        String dataset = e as String;
        return Map.fromIterable(
          dataset.split(_splitTable).last.split(_splitData),
          key: (e) {
            String key = e as String;
            return Methods.toInt(key.split(_splitValue).first);
          },
          value: (e) {
            String rangeValue = e as String;
            List<String> splits = rangeValue.split(_splitValue);
            double start = Methods.toDouble(splits[1]);
            double end = Methods.toDouble(splits[2]);
            String title = splits[3];
            return RangeValue(start, end, title: title);
          },
        );
      },
    );
    if (kDebugMode) {
      print('//===========================');
      print('load Range from local success');
      Common.mapRange.forEach((key, value) {
        if (kDebugMode) {
          print('||$key: ${value.length} item');
        }
      });
      print('\\\\===========================');
    }
  }
}

==> 0.4.92.tar.gz: lib/picker_range.dart <==
import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/model.dart';
import 'package:wowit/sql_service.dart';
import 'common.dart';
import 'methods.dart';
import 'package:collection/collection.dart';

class PickerRange {
  PickerRange._();
  static const String _picker = 'picker';
  static const String _range = 'range';

  static const String _splitTable = '[::]';
  static const String _splitValue = '[..]';
  static const String _splitData = '[,,]';

  static KeyValue getDefaultPicker({required String tableName}) {
    if (kDebugMode) {
      return KeyValue(
          -1, -1, 'Chưa nhập liệu cho drop dơn này: ' + tableName.toString());
    } else {
      return const KeyValue(-1, -1, '--');
    }
  }

  static init(
    Map<String, List<Map<String, Object?>?>?> reponse, {
    bool reload = false,
  }) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> pickers = prefs.getStringList(_picker) ?? [];
    List<String> ranges = prefs.getStringList(_range) ?? [];
    if (pickers.isEmpty || ranges.isEmpty) {
      reload = true;
    }
    if (reload) {
      try {
        if (reponse.isEmpty) {
          reponse = await SQLService.execute('sp_InitSetting',
              params: {'@getPickerRange': true});
          if (reponse.isEmpty) {
            return;
          }
        }
        reponse.forEach((key, value) {
          String tableName = key;
          if (kDebugMode) {
            print('Picker-Range init: ' + tableName);
          }
          if (tableName.startsWith('DataRange_')) {
            PickerRange.initDataRange(
                tableName.replaceAll('DataRange_', ''), value,
                override: true);
          } else if (tableName.startsWith('DataPicker_')) {
            PickerRange.initDataPicker(
                tableName.replaceAll('DataPicker_', ''), value,
                override: true);
          }
        });
        savePickerToLocal();
        saveRangeToLocal();
        await SQLService.execute('sp_updatePicker');
        return;
      } catch (ex) {
        if (kDebugMode) {
          print('PickerRange init exception: ' + ex.toString());
        }
      }
    } else {
      loadPickerFromLocal(pickers: pickers);
      loadRangeFromLocal(ranges: ranges);
    }
  }

  static String getTitle(
    String tableName,
    int? id, {
    String defaultValue = '',
  }) {
    if (id == null) return defaultValue;
    return getDataPicker(
      getValueDataPicker(tableName),
      id,
      defaultValue: defaultValue,
    );
  }

  static String getDataPicker(
    List<KeyValue> data,
    int? id, {
    String defaultValue = '',
  }) {
    if (id == null) return defaultValue;
    return data.firstWhereOrNull((e) => e.valueMember == id)?.displayMember ??
        defaultValue;
    // for (int i = 0; i < data.length; i++) {
    //   try {
    //     if (data[i].valueMember == id) {
    //       return data[i].displayMember;
    //     }
    //   } catch (e) {
    //     if (kDebugMode) {
    //       HandleException.log(e, subTitle: 'getDataPicker');
    //       return defaultValue;
    //     }
    //   }
    // }

    // return defaultValue;
  }

  static List<KeyValue> getValueDataPicker(
    String tableName, {
    int? parentId,
  }) {
    if (Common.mapPicker.containsKey(tableName)) {
      if (parentId == null || parentId < 1) {
        return Common.mapPicker[tableName] ?? [];
      }
      List<KeyValue> data = Common.mapPicker[tableName] ?? [];
      List<KeyValue> lst = [];
      for (int i = 0; i < data.length; i++) {
        if (data[i].parentValue == parentId) {
          lst.add(data[i]);
        }
      }
      if (lst.isEmpty) {
        lst.add(PickerRange.getDefaultPicker(tableName: tableName));
        if (kDebugMode) {
          SQLService.execute('sp_AddPickerRange',
              params: {'@tableName': tableName});
        }
      }
      return lst;
    } else {
      if (kDebugMode) {
        SQLService.execute(
          'sp_AddPickerRange',
          params: {
            '@tableName': tableName,
          },
        );
      }
    }
    return [
      PickerRange.getDefaultPicker(tableName: tableName),
    ];
  }

  static Future<List<KeyValue>> initDataPicker(
    String tableName,
    List<Map<String, Object?>?>? dtData, {
    bool override = false,
  }) async {
    if (Common.mapPicker.containsKey(tableName) && !override) {
      return Common.mapPicker[tableName] ?? [];
    }
    List<KeyValue> listVal = [];
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute(
          'sp_getValueDataPicker',
          params: {
            '@tableName': tableName,
          },
        );
        if (ds.isEmpty) return listVal;
        dtData = ds['sp_getValueDataPicker'] ?? ds.values.first;
      }
      dtData ??= [];

      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        int parentId = Methods.getInt(row, 'parentId');
        String title = Methods.getString(row, 'title');
        if (id < 0) continue;
        listVal.add(KeyValue(parentId, id, title));
      }
      Common.mapPicker[tableName] = listVal;
    } catch (ex) {
      if (kDebugMode) {
        print('[Exception] getValueDataPicker: ' + ex.toString());
      }
    }
    if (kDebugMode) {
      print('getValueDataPicker: ' +
          tableName +
          ' count: ' +
          listVal.length.toString());
    }
    return listVal;
  }

  static Map<int, RangeValue> getValueDataRange(String tableName) {
    if (Common.mapRange.containsKey(tableName)) {
      return Common.mapRange[tableName] ?? {};
    } else if (kDebugMode) {
      SQLService.execute('sp_AddPickerRange',
          params: {'@tableName': tableName});
    }
    return {};
  }

  static Future<Map<int, RangeValue>> initDataRange(
    String tableName,
    List<Map<String, Object?>?>? dtData, {
    bool override = false,
  }) async {
    if (Common.mapRange.containsKey(tableName) && !override) {
      return Common.mapRange[tableName] ?? {};
    }
    Map<int, RangeValue> listVal = {};
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute('sp_getValueDataRange',
            params: {'@tableName': tableName});
        if (ds.isEmpty) return listVal;
        dtData = ds['sp_getValueDataRange'] ?? ds.values.first;
      }
      dtData ??= [];
      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        String title = Methods.getString(row, 'title');
        double from = Methods.getDouble(row, 'valFrom', defaultValue: 0);
        double to = Methods.getDouble(row, 'valTo', defaultValue: 0);
        if (id < 0) continue;
        listVal[id] = RangeValue(from, to, title: title);
      }
      Common.mapRange[tableName] = listVal;
    } catch (ex) {
      if (kDebugMode) {
        print('[Exception] initDataRange: ' + ex.toString());
      }
    }
    if (kDebugMode) {
      print('getValueDataPicker: ' +
          tableName +
          ' count: ' +
          listVal.length.toString());
    }
    return listVal;
  }

  static RangeValue getRangeValueDataPicker(
      Map<int, RangeValue> rangeMap, int id,
      {RangeValue defaultValue = const RangeValue(0, 0)}) {
    return rangeMap[id] ?? defaultValue;
  }

  static void savePickerToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> pickers = Common.mapPicker.entries.map<String>((e) {
      return e.key +
          _splitTable +
          e.value.map<String>((e) {
            String parentValue = e.parentValue.toString();
            String valueMember = e.valueMember.toString();
            String displayMember =
                e.displayMember.isNotEmpty ? e.displayMember : ' ';
            return parentValue +
                _splitValue +
                valueMember +
                _splitValue +
                displayMember;
          }).join(_splitData);
    }).toList();

    await prefs.setStringList(_picker, pickers);
    if (kDebugMode) {
      print('save Picker to local success');
    }
  }

  static void saveRangeToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> ranges = Common.mapRange.entries.map<String>((e) {
      return e.key +
          _splitTable +
          e.value.entries.map<String>((e) {
            String id = e.key.toString();
            String start = e.value.start.toString();
            String end = e.value.start.toString();
            String title = e.value.title;
            title = title.isNotEmpty ? title : ' ';
            return id +
                _splitValue +
                start +
                _splitValue +
                end +
                _splitValue +
                title;
          }).join(_splitData);
    }).toList();

    await prefs.setStringList(_range, ranges);
    if (kDebugMode) {
      print('save Range to local success');
    }
  }

  static void loadPickerFromLocal({List<String>? pickers}) async {
    if (pickers == null || pickers.isEmpty) {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      pickers = prefs.getStringList(_picker) ?? [];
    }
    Common.mapPicker = Map.fromIterable(
      pickers,
      key: (e) {
        String key = e as String;
        return key.split(_splitTable).first;
      },
      value: (e) {
        String dataset = e as String;
        return dataset.split(_splitTable).last.split(_splitData).map((e) {
          List<String> splits = e.split(_splitValue);
          int parentValue = Methods.toInt(splits[0]);
          int valueMember = Methods.toInt(splits[1]);
          String displayMember = splits[2];
          return KeyValue(parentValue, valueMember, displayMember);
        }).toList();
      },
    );
    if (kDebugMode) {
      print('//==============================');
      print('||load Picker from local success');
      Common.mapPicker.forEach((key, value) {
        if (kDebugMode) {
          print('||$key: ${value.length} item');
        }
      });
      print('\\\\==============================');
    }
  }

  static void loadRangeFromLocal({List<String>? ranges}) async {
    if (ranges == null || ranges.isEmpty) {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      ranges = prefs.getStringList(_range) ?? [];
    }
    Common.mapRange = Map.fromIterable(
      ranges,
      key: (e) {
        String key = e as String;
        return key.split(_splitTable).first;
      },
      value: (e) {
        String dataset = e as String;
        return Map.fromIterable(
          dataset.split(_splitTable).last.split(_splitData),
          key: (e) {
            String key = e as String;
            return Methods.toInt(key.split(_splitValue).first);
          },
          value: (e) {
            String rangeValue = e as String;
            List<String> splits = rangeValue.split(_splitValue);
            double start = Methods.toDouble(splits[1]);
            double end = Methods.toDouble(splits[2]);
            String title = splits[3];
            return RangeValue(start, end, title: title);
          },
        );
      },
    );
    if (kDebugMode) {
      print('//===========================');
      print('load Range from local success');
      Common.mapRange.forEach((key, value) {
        if (kDebugMode) {
          print('||$key: ${value.length} item');
        }
      });
      print('\\\\===========================');
    }
  }
}
